/* gcc -g -Wall -o seek_avi seek.c `pkg-config --cflags --libs gstreamer-0.10` */

#include <gst/gst.h>

gint    width;
gint    height;
int     fps_num;
int     fps_den;

static void new_decoded_pad_cb (GstElement *demuxer,
                                GstPad     *new_pad,
                                GstElement *pipeline);

static GstElement *
_buid_pipeline_dvd ()
{
  GstElement          *pipeline;
  GstElement          *dvdnavsrc;
  GstElement          *demuxer;
  GstStateChangeReturn res;

  pipeline = gst_pipeline_new ("pipeline");
  if (!pipeline)
    return NULL;

  dvdnavsrc = gst_element_factory_make ("dvdnavsrc", "dvdnavsrc");
  if (!dvdnavsrc)
    goto failure;

  demuxer = gst_element_factory_make ("dvddemux", "dvddemux");
  if (!demuxer)
    goto failure;

  g_signal_connect (demuxer, "pad-added",
                    G_CALLBACK (new_decoded_pad_cb), pipeline);

  gst_bin_add_many (GST_BIN (pipeline),
                    dvdnavsrc, demuxer, NULL);

  if (!gst_element_link (dvdnavsrc, demuxer))
    goto failure;

  res = gst_element_set_state (pipeline, GST_STATE_PAUSED);
  if (res == GST_STATE_CHANGE_FAILURE)
    goto failure;

  res = gst_element_get_state( pipeline, NULL, NULL, GST_CLOCK_TIME_NONE );
  if (res != GST_STATE_CHANGE_SUCCESS)
    goto failure;

  return pipeline;

 failure:
  gst_object_unref (GST_OBJECT (pipeline));
  return NULL;
}

int
main (int argc, char *argv[])
{
  GstElement *pipeline;

  gst_init (&argc, &argv);

  pipeline = _buid_pipeline_dvd ();
  if (!pipeline) {
    g_print ("Error during the pipeline building\n");
    gst_deinit ();
    return -1;
  }

  gst_element_set_state (pipeline, GST_STATE_PLAYING);

  return 0;
}

static void
new_decoded_pad_cb (GstElement *demuxer,
                    GstPad     *new_pad,
                    GstElement *pipeline)
{
  GstPad *pad;
  GstCaps *caps;
  gchar *str;

  g_print ("new pad\n");

  caps = gst_pad_get_caps (new_pad);
  str = gst_caps_to_string (caps);

  g_print ("str: %s\n", str);
  if (g_str_has_prefix (str, "video/mpeg")) {
    GstElement          *decoder;
    GstElement          *conv;
    GstElement          *sink;

    decoder = gst_element_factory_make ("mpeg2dec", "decoder");
    if (!decoder)
      exit (-1);

    conv = gst_element_factory_make ("ffmpegcolorspace", "conv");
    if (!conv)
      exit (-1);

    sink = gst_element_factory_make ("ximagesink", "sink");
    if (!sink)
      exit (-1);

    gst_bin_add_many (GST_BIN (pipeline),
                      decoder, conv, sink, NULL);

    if (!gst_element_link (decoder, conv))
      exit (-1);

    if (!gst_element_link (conv, sink))
      exit (-1);

    pad = gst_element_get_pad (decoder, "sink");
    if (GST_PAD_LINK_FAILED (gst_pad_link (new_pad, pad))) {
      g_warning ("Failed to link %s:%s to %s:%s", GST_DEBUG_PAD_NAME (new_pad),
                 GST_DEBUG_PAD_NAME (pad));
    }
  }
  g_free (str);
  gst_caps_unref (caps);
}
