/* gcc -g -Wall -o seek_avi seek.c `pkg-config --cflags --libs gstreamer-0.10` */

#include <gst/gst.h>

gint    width;
gint    height;
int     fps_num;
int     fps_den;

static void new_decoded_pad_cb (GstElement *demuxer,
                                GstPad     *new_pad,
                                gpointer    user_data);

static GstElement *
_buid_pipeline_avi (gchar *filename)
{
  GstElement          *pipeline;
  GstElement          *filesrc;
  GstElement          *demuxer;
  GstElement          *decoder;
  GstElement          *sink;
  GstStateChangeReturn res;

  pipeline = gst_pipeline_new ("pipeline");
  if (!pipeline)
    return NULL;

  filesrc = gst_element_factory_make ("filesrc", "filesrc");
  if (!filesrc)
    goto failure;
  g_object_set (G_OBJECT (filesrc), "location", filename, NULL);

  demuxer = gst_element_factory_make ("avidemux", "demuxer");
  if (!demuxer)
    goto failure;

/*   decoder = gst_element_factory_make ("xviddec", "decoder"); */
  decoder = gst_element_factory_make ("ffdec_mpeg4", "decoder");
  if (!decoder)
    goto failure;

  g_signal_connect (demuxer, "pad-added",
                    G_CALLBACK (new_decoded_pad_cb), decoder);

  sink = gst_element_factory_make ("avs3sink", "sink");
  if (!sink)
    goto failure;

  gst_bin_add_many (GST_BIN (pipeline),
                    filesrc, demuxer, decoder, sink, NULL);

  if (!gst_element_link (filesrc, demuxer))
    goto failure;
  if (!gst_element_link (decoder, sink))
    goto failure;

  res = gst_element_set_state (pipeline, GST_STATE_PAUSED);
  if (res == GST_STATE_CHANGE_FAILURE)
    goto failure;

  res = gst_element_get_state( pipeline, NULL, NULL, GST_CLOCK_TIME_NONE );
  if (res != GST_STATE_CHANGE_SUCCESS)
    goto failure;

  return pipeline;

 failure:
  gst_object_unref (GST_OBJECT (pipeline));
  return NULL;
}

static void
_seek (GstElement *pipeline, int frame_nbr)
{
  GstElement *sink;
  GstBuffer  *buffer = NULL;

  sink = gst_bin_get_by_name (GST_BIN (pipeline), "sink");
  g_signal_emit_by_name (G_OBJECT (sink), "seek", frame_nbr, &buffer);
}

int
main (int argc, char *argv[])
{
  GstElement *pipeline;
  gchar      *filename;
  gint i;

  gst_init (&argc, &argv);

  /* arguments */

  if (argc < 2) {
    g_print ("usage: %s file.avi\n", argv[0]);
    exit (-1);
  }
  filename = argv[1];

  pipeline = _buid_pipeline_avi (filename);
  if (!pipeline) {
    g_print ("Error during the pipeline building\n");
    gst_deinit ();
    return -1;
  }


  _seek (pipeline, 0);
  _seek (pipeline, 1);
  _seek (pipeline, 2);
  _seek (pipeline, 3);
  _seek (pipeline, 4);
  for (i = 0; i < 15; i++)
    {
      g_print ("i : %d\n", i);
      _seek (pipeline, i*100);
    }

/*   _display_info (pipeline); */

  return 0;
}

static void
new_decoded_pad_cb (GstElement *demuxer,
                    GstPad     *new_pad,
                    gpointer    user_data)
{
  GstElement *decoder;
  GstPad *pad;
  GstCaps *caps;
  gchar *str;

  g_print ("new pad\n");

  caps = gst_pad_get_caps (new_pad);
  str = gst_caps_to_string (caps);

  if (g_str_has_prefix (str, "video/")) {
    decoder = GST_ELEMENT (user_data);

    pad = gst_element_get_pad (decoder, "sink");
    if (GST_PAD_LINK_FAILED (gst_pad_link (new_pad, pad))) {
      g_warning ("Failed to link %s:%s to %s:%s", GST_DEBUG_PAD_NAME (new_pad),
                 GST_DEBUG_PAD_NAME (pad));
    }
  }
  g_free (str);
  gst_caps_unref (caps);
}
